
/**
 * @author dhan
 *
 * java BillUserRequest <merchantpayment_id>
 *
 */

import java.net.URL;
import org.apache.axis.message.SOAPHeaderElement;
import org.apache.axis.message.PrefixedQName;
import org.apache.axis.message.MessageElement;
import org.apache.axis.AxisFault;
import org.apache.axis.client.*;
import java.io.*;
import java.util.*;
import com.paypal.api.client.*;        //PayPal Library, See the src directory.
import eBLBaseComponents.apis.ebay.*;  // eBay Business Logic definition, from eBL_* wsdl.
import PayPalAPI.api.ebay.*;  // Based on PayPalSvc.wsdl.
import CoreComponentTypes.apis.ebay.*;


public class BillUserRequest 
{
    public static void main(String[] args) throws Exception 
    {
        if (args.length < 1)
        {
                System.out.println("Usage: java BillUserRequest <MP_ID> [<MEMO>]");
                return;
        }

        BillUserRequest client = new BillUserRequest();
        ReadArgs a = new ReadArgs("h:s u:s p:s k:s w:s v:s",args);
        if (!a.ok()) usage();
        Hashtable h = a.hash();
        String[] argv = a.args();
        if (argv.length>2 || argv.length<1) {
            System.out.println("Usage: java BillUserRequest <MP_ID> [<MEMO>]");
            return;
        }
        String memo = null;
        if (argv.length==2) {memo=argv[1];}
        SimpleConfigFile cfg = new SimpleConfigFile("client_config.nvp");

        if (a.contains("h")) {cfg.set("url",(String)a.val("h"));}
        if (a.contains("u")) {cfg.set("user_name",(String)a.val("u"));}
        if (a.contains("p")) {cfg.set("user_passwd",(String)a.val("p"));}
        if (a.contains("k")) {cfg.set("cert_file",(String)a.val("k"));}
        if (a.contains("w")) {cfg.set("cert_password",(String)a.val("w"));}
        if (a.contains("v")) {cfg.set("mp_version",(String)a.val("v"));}
        client.runBillUserRequest(argv[0],memo,cfg);
    }

    public BillUserRequest() {}

    public static void usage() {
        System.out.println("usage: java <action> [options] <txn-id>");
        System.out.println("  Options:");
        System.out.println("     -f config-file   Default: client_config.nvp in current directory.");
        System.out.println("                      Command line options override this file.");
        System.out.println("     -h url");
        System.out.println("     -u username");
        System.out.println("     -p password");
        System.out.println("     -k keystore-file (as in *.p12, aka PKCS12)");
        System.out.println("     -w keystore-paswd");
        //System.out.println("     -v version (as in "1.0" -- currently not supported by axia 1.1)");
        System.out.println("     -v version ");
        System.out.println("     -e ");
        System.out.println("  actions:");
        System.out.println("     RefundTransaction <transaction-id> [<memo>]");
        System.out.println("     GetTxnDetails <transaction-id>");
        System.out.println("     BillUserRequest <merchantpayment_id> [<memo>]");
        System.exit(1);
    }

    public void runBillUserRequest(String mp_id, String memo, SimpleConfigFile cfg) throws Exception {
        // First, we load the config file, and build a request object...
        String url = cfg.val("url");
        System.out.println("connecting to \""+url+"\": testing MerchantPullPayment(\""+mp_id+"\")");

        PayPalAPIInterface binding = null;
        try {
            binding = new PayPalAPIInterfaceServiceLocator().getPayPalAPI(new URL(url));
            ((PayPalAPISoapBindingStub)binding).setTimeout(60000);
        }
        catch(Exception e) {
	    org.apache.axis.AxisFault afe = null;
	    if (e instanceof org.apache.axis.AxisFault) {
		afe = (org.apache.axis.AxisFault) e;
	    }
	    String note = afe.getFaultString();
	    System.out.println("Got AxisFault exception " + note.toString());
	    e.printStackTrace();
           // throw AxisFault.makeFault(e);
	    return;
        }

	// start composing the request  object
        MerchantPullPaymentType payment = new MerchantPullPaymentType();
	
	payment.setMpID(new eBLBaseComponents.apis.ebay.MerchantPullIDType(mp_id));
	String amount = cfg.val("mp_amount");
	String currencyID = cfg.val("mp_currency");
	if (amount.length() != 0) {
	    CoreComponentTypes.apis.ebay.BasicAmountType amountType = new CoreComponentTypes.apis.ebay.BasicAmountType(amount);
	    payment.setAmount(amountType);

	    if (currencyID.length() != 0)
		amountType.setCurrencyID(eBLBaseComponents.apis.ebay.CurrencyCodeType.fromString(currencyID));
	}

	String paymentType = cfg.val("mp_paymenttype");
	if (paymentType.length() != 0) {
	    payment.setPaymentType(eBLBaseComponents.apis.ebay.MerchantPullPaymentCodeType.fromString(paymentType));
	}	
	if (memo == null) {
	    memo = cfg.val("mp_memo");
	}
	if (memo.length() != 0)
	    payment.setMemo(memo);

	String emailSubject = cfg.val("mp_emailSubject");
	if (emailSubject.length() != 0)
	    payment.setEmailSubject(cfg.val("mp_emailSubject"));

	String tax = cfg.val("mp_tax_amount");
	if (tax.length() != 0) {
	    CoreComponentTypes.apis.ebay.BasicAmountType taxAmount = new CoreComponentTypes.apis.ebay.BasicAmountType(tax);
	    payment.setTax(taxAmount);
	}
	
	String shipping = cfg.val("mp_shipping_amount");
	if (shipping.length() != 0) {
	     CoreComponentTypes.apis.ebay.BasicAmountType shpAmount = new CoreComponentTypes.apis.ebay.BasicAmountType(shipping);
	    payment.setShipping(shpAmount);
	}

	String handlinging = cfg.val("mp_handling_amount");
	if (handlinging.length() != 0) {
	    CoreComponentTypes.apis.ebay.BasicAmountType hdlAmount = new CoreComponentTypes.apis.ebay.BasicAmountType(handlinging);
	    payment.setHandling(hdlAmount);
	}

	String itemName = cfg.val("mp_itemName");
	if (itemName.length() != 0)
	    payment.setItemName(itemName);
	String itemNumber = cfg.val("mp_itemNumber");
	if (itemNumber.length() != 0)
	    payment.setItemNumber(itemNumber);
	String invoice = cfg.val("mp_invoice");
	if (invoice.length() != 0)
	    payment.setInvoice(invoice);
	String custom = cfg.val("mp_custom");
	if (custom.length() != 0)
	    payment.setCustom(custom);
	
	BillUserRequestType request = new BillUserRequestType();	
        request.setVersion(cfg.val("mp_version")); // A bug in axis 1.1 causes this to fail.
	request.setMerchantPullPaymentDetails(payment);
	
        _BillUserReq _req = new _BillUserReq();
        _req.setBillUserRequest(request);


        // Use proxy settings if set in config file...
        if (cfg.val("proxy_host") != null) {
            System.setProperty("http.proxyHost",cfg.val("proxy_host"));
            if (cfg.val("proxy_host") != null) {
                System.setProperty("http.proxyPort",cfg.val("proxy_port"));
            } else {
                System.setProperty("http.proxyPort","80");
            }
        }

	// Insert ebl security ...
	PrefixedQName pqn = new PrefixedQName("urn:ebay:api:PayPalAPI",
                                                "RequesterCredentials", "");
	SOAPHeaderElement eblSecurity = new SOAPHeaderElement(pqn);
	eblSecurity.setMustUnderstand(true);

        MessageElement credentials = new MessageElement("Credentials", "ebl", "urn:ebay:apis:eBLBaseComponents");
	MessageElement username = new MessageElement("", "ebl:Username");
        MessageElement password = new MessageElement("", "ebl:Password");
        MessageElement subject = new MessageElement("", "ebl:Subject");
        username.setObjectValue(cfg.val("user_name"));
        credentials.addChild(username);
        password.setObjectValue(cfg.val("user_passwd"));
        credentials.addChild(password);
        subject.setObjectValue(cfg.val("user_subject"));
        credentials.addChild(subject);
        eblSecurity.addChild(credentials);
        org.apache.axis.client.Stub s = (Stub)binding;
        s.setHeader(eblSecurity);

        // Below allows SSL connections...
        org.apache.axis.AxisProperties.setProperty("axis.socketSecureFactory",
                                                   "com.paypal.api.client.SSLFactory");

	//Print the request object
	printBillUserReq(request);

        // This actually does the transaction ...
	BillUserResponseType resp = null;
	try {
        	resp = binding.billUser(_req);
	} catch(Exception e) {
		org.apache.axis.AxisFault afe = null;
		if (e instanceof org.apache.axis.AxisFault){
			afe = (org.apache.axis.AxisFault)e;
		}
		String note = afe.dumpToString();
		System.out.println("Failed on transaction: " + note.toString());	
		//e.printStackTrace();
		return;
	}

	//process the response...
	printBillUserRes(resp);

    }

    public void printBillUserReq(BillUserRequestType req) {
	MerchantPullPaymentType payment = req.getMerchantPullPaymentDetails();

        System.out.println();
        System.out.println("MerchantPullPayment Request ---------");
        System.out.println("Merchant payment ID:	" + payment.getMpID().toString());
	if (payment.getAmount() != null)
            System.out.println("Request amount:		" + payment.getAmount().toString());

	if (payment.getPaymentType() != null)
            System.out.println("PaymentType:		" + payment.getPaymentType().toString());
	if (payment.getMemo() != null)
	    System.out.println("Memo:			" + payment.getMemo());
	if (payment.getEmailSubject() != null)
	    System.out.println("EmailSubject:		" + payment.getEmailSubject());
	if (payment.getTax() != null)
	    System.out.println("Tax:			" + payment.getTax().toString());
	if (payment.getShipping() != null)
	    System.out.println("Shipping:		" + payment.getShipping().toString());
	if (payment.getHandling() != null)
	    System.out.println("Handling:		" + payment.getHandling().toString());
	if (payment.getItemName() != null)
	    System.out.println("Item name:		" + payment.getItemName());
	if (payment.getItemNumber() != null)
	    System.out.println("Item number:		" + payment.getItemNumber());
	if (payment.getInvoice() != null)
	    System.out.println("Invoice:		" + payment.getInvoice());
	if (payment.getCustom() != null)
	    System.out.println("Custom:			" + payment.getCustom());
	
    }

    public void printBillUserRes(BillUserResponseType res) {
	MerchantPullPaymentResponseType response = res.getBillUserResponseDetails();

        // Process the response...
        System.out.println();
        System.out.println("MerchantPullPayment Response ---------");
        System.out.println("Ack:		" + res.getAck().toString());
        System.out.println("TimeStamp:		" + res.getTimestamp().getTime());
	if (res.getCorrelationID() != null)
       	    System.out.println("CorrelationID:	" + res.getCorrelationID().toString());
	if (res.getVersion() != null)
            System.out.println("Version:		" + res.getVersion().toString());
	if (res.getBuild() != null)
            System.out.println("Build:		" + res.getBuild().toString());

	PrintResults display = new PrintResults();	

	if (response.getPayerInfo() != null) {
	    System.out.println("	************  Payer Info");	
	    display.printPayerInfoType(response.getPayerInfo());
	}
	
	if (response.getPaymentInfo() != null) { 
	    System.out.println("	************  Payment Info");	
	    display.printPaymentInfo(response.getPaymentInfo()); 
	}

	if (response.getMerchantPullInfo() != null) {
	    System.out.println("	************  Merchant Pull Info");	
  	    display.printMerchantPullInfoType(response.getMerchantPullInfo());	
	}
	
        eBLBaseComponents.apis.ebay.ErrorType err[] = res.getErrors();
        for (int i=0; err!=null&&i<err.length; i++) {
            System.out.println();
            System.out.println("Error Code: " + err[i].getErrorCode());
            System.out.println("Short mssg: " + err[i].getShortMessage());
            System.out.println("Long messg: " + err[i].getLongMessage());
            System.out.println("Severity:   " + err[i].getSeverityCode());
	}
    }
}

