
/**
 * @author dhan
 *
 * java MassPay <File_Path>
 *
 */

import java.net.URL;
import org.apache.axis.message.SOAPHeaderElement;
import org.apache.axis.message.PrefixedQName;
import org.apache.axis.message.MessageElement;
import org.apache.axis.AxisFault;
import org.apache.axis.client.*;
import java.io.*;
import java.util.*;
import com.paypal.api.client.*;        //PayPal Library, See the src directory.
import eBLBaseComponents.apis.ebay.*;  // eBay Business Logic definition, from eBL_* wsdl.
import PayPalAPI.api.ebay.*;  // Based on PayPalSvc.wsdl.
import CoreComponentTypes.apis.ebay.*;


public class MassPay 
{
    public static void main(String[] args) throws Exception 
    {
        if (args.length < 2)
        {
                System.out.println("Usage: java MassPay <Num_Entries> <File_Path> [-sub EmailSubject] ");
                return;
        }
        //
        MassPay client = new MassPay();
        ReadArgs a = new ReadArgs("h:s u:s p:s k:s w:s v:s sub:s cc:s",args);
        if (!a.ok()) usage();
        Hashtable h = a.hash();
        String[] argv = a.args();
        if (argv.length>2 || argv.length<1) {
            System.out.println("Usage: java MassPay <Num_Entries> <File-Path> ");
            return;
        }

        SimpleConfigFile cfg = new SimpleConfigFile("client_config.nvp");

        if (a.contains("h")) {cfg.set("url",(String)a.val("h"));}
        if (a.contains("u")) {cfg.set("user_name",(String)a.val("u"));}
        if (a.contains("p")) {cfg.set("user_passwd",(String)a.val("p"));}
        if (a.contains("k")) {cfg.set("cert_file",(String)a.val("k"));}
        if (a.contains("w")) {cfg.set("cert_password",(String)a.val("w"));}
		if (a.contains("v")) {cfg.set("massp_version", (String)a.val("v"));}
        if (a.contains("sub")) {cfg.set("massp_email_subj",(String)a.val("sub"));}
        client.runMassPay(argv[0], argv[1], cfg);
    }

    public MassPay() {}

    public static void usage() {
        System.out.println("usage: java <action> [options] <file_path>");
        System.out.println("  Options:");
        System.out.println("     -f config-file   Default: client_config.nvp in current directory.");
        System.out.println("                      Command line options override this file.");
        System.out.println("     -h url");
        System.out.println("     -u username");
        System.out.println("     -p password");
        System.out.println("     -k keystore-file (as in *.p12, aka PKCS12)");
        System.out.println("     -w keystore-paswd");
        //System.out.println("     -v version (as in "1.0" -- currently not supported by axia 1.1)");
        System.out.println("     -v version ");
        System.out.println("     -sub subject");
        System.out.println("     -e ");
        System.out.println("  actions:");
        System.out.println("     RefundTransaction <transaction-id> [<memo>]");
        System.out.println("     GetTxnDetails <transaction-id>");
        System.out.println("     MassPay <File_Path>");
        System.exit(1);
    }

    public void runMassPay(String num_entries, String file_path, SimpleConfigFile cfg) throws Exception {
        // First, we load the config file, and build a request object...
        String url = cfg.val("url");
        System.out.println("connecting to \""+url+"\": testing MassPay with testfile(\""+file_path+"\")");
	
        //PpapiPortType binding = null;
        PayPalAPIInterface binding = null;
        try {
            binding = new PayPalAPIInterfaceServiceLocator().getPayPalAPI(new URL(url));
            ((PayPalAPISoapBindingStub)binding).setTimeout(60000);
        }
        catch(Exception e) {
	    org.apache.axis.AxisFault afe = null;
	    if (e instanceof org.apache.axis.AxisFault) {
		afe = (org.apache.axis.AxisFault) e;
	    }
	    String note = afe.getFaultString();
	    System.out.println("Got AxisFault exception " + note);
	    e.printStackTrace();
           // throw AxisFault.makeFault(e);
	    return;
        }

        MassPayRequestType request = new MassPayRequestType();
        _MassPayReq _req = new _MassPayReq();
	
	setMassPayReq(num_entries, file_path, cfg, request);
        _req.setMassPayRequest(request);

	printMassPayReq(request);


        // Use proxy settings if set in config file...
        if (cfg.val("proxy_host") != null) {
            System.setProperty("http.proxyHost",cfg.val("proxy_host"));
            if (cfg.val("proxy_host") != null) {
                System.setProperty("http.proxyPort",cfg.val("proxy_port"));
            } else {
                System.setProperty("http.proxyPort","80");
            }
        }

	// Insert ebl security ...
	PrefixedQName pqn = new PrefixedQName("urn:ebay:api:PayPalAPI", "RequesterCredentials", "");
	SOAPHeaderElement eblSecurity = new SOAPHeaderElement(pqn);	
	eblSecurity.setMustUnderstand(true);
	
	MessageElement credentials = new MessageElement("Credentials", "ebl", "urn:ebay:apis:eBLBaseComponents");

	MessageElement username = new MessageElement("", "ebl:Username");
	MessageElement password = new MessageElement("", "ebl:Password");
	MessageElement subject = new MessageElement("", "ebl:Subject");
	username.setObjectValue(cfg.val("user_name"));
	credentials.addChild(username);
	password.setObjectValue(cfg.val("user_passwd"));
	credentials.addChild(password);
	subject.setObjectValue(cfg.val("user_subject"));
	credentials.addChild(subject);
	eblSecurity.addChild(credentials);
	org.apache.axis.client.Stub s = (Stub)binding;
	s.setHeader(eblSecurity);
	

        // Below allows SSL connections...
        org.apache.axis.AxisProperties.setProperty("axis.socketSecureFactory",
                                                   "com.paypal.api.client.SSLFactory");

        // This actually does the transaction ...
	PayPalAPI.api.ebay.MassPayResponseType resp = null;
	try {
        	resp = binding.massPay(_req);
	} catch(Exception e) {
		org.apache.axis.AxisFault afe = null;
		if (e instanceof org.apache.axis.AxisFault){
			afe = (org.apache.axis.AxisFault)e;
		}
		String note = afe.dumpToString();
		System.out.println("Failed on transaction: " + note);	
		//e.printStackTrace();
		return;
	}

	//process the response...
	printMassPayResp(resp);

    }

    public void printMassPayReq(MassPayRequestType req) {
        System.out.println();
        System.out.println("MassPay Request ---------");

	PrintResults display = new PrintResults();
	
	if (req.getEmailSubject() != null)
       	    System.out.println("Email Subject:	" + req.getEmailSubject());

	PayPalAPI.api.ebay.MassPayRequestItemType[] items = req.getMassPayItem();
	for (int i=0; items!=null && i<items.length; i++){
       	    System.out.println("---- Payment Items [" + i + "]: ----");
	    display.printMassPayItemType(items[i]);
	} 
    }

    public int setMassPayReq(String num_entries, String file_path, SimpleConfigFile cfg, MassPayRequestType req) {
	int Max_entry = Integer.parseInt(num_entries);
	PayPalAPI.api.ebay.MassPayRequestItemType[] items = new PayPalAPI.api.ebay.MassPayRequestItemType[Max_entry];

	req.setVersion(cfg.val("massp_version"));
	String emailsub = new String(cfg.val("massp_email_subj"));
	if (emailsub.length() !=0)
	    req.setEmailSubject(emailsub);

	DataInputStream dis = null;
	String record = null;
	int recCount = 0;

	try {
	    File f = new File(file_path);
	    FileInputStream fis = new FileInputStream(f);
	    BufferedInputStream bis = new BufferedInputStream(fis);
	    dis = new DataInputStream(bis);

	    while ((record=dis.readLine()) != null && (recCount <= Max_entry)) {
		
		String[] temp = null;
		temp = record.split("\t");

		items[recCount]  = new PayPalAPI.api.ebay.MassPayRequestItemType();

		EmailAddressType email = new EmailAddressType(temp[0]);
		items[recCount].setReceiverEmail(email);

		if (temp[1].length() != 0) {
	    	BasicAmountType amount = new BasicAmountType(temp[1]);
	  		amount.setCurrencyID(eBLBaseComponents.apis.ebay.CurrencyCodeType.fromString(temp[2]));

			items[recCount].setAmount(amount);
		}
		items[recCount].setUniqueId(temp[3]);
		items[recCount].setNote(temp[4]);

		recCount++;
	    }
	} catch (IOException e) {
	    System.out.println("IOException error!" + e.getMessage());
	} finally {
	    if (dis != null) {
	    	try {
		    dis.close();
	    	}catch (IOException ioe) {
	    	}
	    }
	}

	req.setMassPayItem(items);

	return 0;
	
    }

    public void printMassPayResp(PayPalAPI.api.ebay.MassPayResponseType res) {
        // Process the response...
        System.out.println();
        System.out.println("MassPay Response ---------");
        System.out.println("Ack:		" + res.getAck().toString());
        System.out.println("TimeStamp:		" + res.getTimestamp().getTime());
	if (res.getCorrelationID() != null)
       	    System.out.println("CorrelationID:	" + res.getCorrelationID().toString());
	if (res.getVersion() != null)
            System.out.println("Version:		" + res.getVersion().toString());
	if (res.getBuild() != null)
            System.out.println("Build:		" + res.getBuild().toString());

        eBLBaseComponents.apis.ebay.ErrorType err[] = res.getErrors();
        for (int i=0; err!=null&&i<err.length; i++) {
            System.out.println();
            System.out.println("Error Code: " + err[i].getErrorCode());
            System.out.println("Short mssg: " + err[i].getShortMessage());
            System.out.println("Long messg: " + err[i].getLongMessage());
            System.out.println("Severity:   " + err[i].getSeverityCode());
	}
    }
}

